indexing
	description: "Names inside IDENTITIES class.%
		% This class is singleton. To use it inherit from CI_IDENTITIES_NAMES_CLIENT"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_IDENTITIES_NAMES

inherit
	CI_SHARED_TOKEN_CONSTANTS

create {CI_IDENTITIES_NAMES_CLIENT}
	default_create

feature -- Classes
	default_identities_name: STRING is
			-- Default name of "IDENTITIES" class
		once
			Result := "IDENTITIES"
		end

	identities_name: ET_CLASS_NAME
			-- Name of "IDENTITIES" class

	generate_identities_name (a_universe: ET_UNIVERSE) is
			-- Generate name of "IDENTITIES" class that is unique within `a_universe'
		require
			a_universe_exists: a_universe /= Void
		once
			from
				create {ET_IDENTIFIER} identities_name.make (default_identities_name)
			until
				not a_universe.has_class (identities_name)
			loop
				identities_name.name.append_character ('_')
			end
		ensure
			identities_name_exists: identities_name /= Void
			identities_name_unique: not a_universe.has_class (identities_name)
		end

	parent_name: ET_CLASS_NAME is
			-- Name of "IDENTITIES"' parent (`INTERNAL' class)
		once
			Result := tokens.internal_class_name
		end

feature -- Features
	identity_name (a_class_name: ET_CLASS_NAME): ET_FEATURE_NAME is
			-- Name of "identity" function for `a_class_name'
		require
			a_class_name_exists: a_class_name /= Void
		do
			create {ET_IDENTIFIER} Result.make (a_class_name.lower_name + "_identity")
		ensure
			result_exists: Result /= Void
			is_unique: not Result.same_feature_name (unknown_identity_name) and
				not Result.same_feature_name (is_expanded_name) and
				not Result.same_feature_name (is_printable_name) and
				not Result.same_feature_name (field_by_name_name)
		end

	unknown_identity_name: ET_FEATURE_NAME is
			-- Name of "identity" function for unknown type (e.g. open formal parameter)
		once
			create {ET_IDENTIFIER} Result.make ("unknown_identity_")
		end

	printable_hash_code_name: ET_FEATURE_NAME is
			-- Name of "printable_hash_code" function
		once
			create {ET_IDENTIFIER} Result.make ("printable_hash_code")
		end

	identity_target_name (a_class_name: ET_CLASS_NAME): ET_FEATURE_NAME is
			-- Name of query that should be included in instrumented class called `a_class_name'
			-- to be a target of calls to "IDENTITIES" features
		do
			create {ET_IDENTIFIER} Result.make (a_class_name.lower_name + "_identities_")
		end

	is_identity_target (a_name: ET_FEATURE_NAME; a_class_name: ET_CLASS_NAME): BOOLEAN is
			-- Is `a_name' the name of identitity target feature in class called `a_class_name'?
		do
			Result := a_name.same_feature_name (identity_target_name (a_class_name))
		end

feature {CI_IDENTITIES_FEATURES} -- Features
	is_expanded_name: ET_FEATURE_NAME is
			-- Name of "is_expanded" feature
		once
			create {ET_IDENTIFIER} Result.make ("is_expanded")
		end

	is_printable_name: ET_FEATURE_NAME is
			-- Name of "is_printable" feature
		once
			create {ET_IDENTIFIER} Result.make ("is_printable")
		end

	field_by_name_name: ET_FEATURE_NAME is
			-- Name of "field_by_name" feature
		once
			create {ET_IDENTIFIER} Result.make ("field_by_name")
		end


feature {CI_IDENTITIES_FEATURES} -- Formal arguments
	class_name_argument: ET_IDENTIFIER is
			-- Name of formal argument for "is_expanded" and "is_printable"
		once
			create Result.make ("c")
		end

	object_argument: ET_IDENTIFIER is
			-- Name of first formal argument for "field_by_name"
		once
			create Result.make ("obj")
		end

	field_name_argument: ET_IDENTIFIER is
			-- Name of second formal argument for "field_by_name"
		once
			create Result.make ("name")
		end

	item_argument: ET_IDENTIFIER is
			-- Name of formal argument for "printable_hash_code"
		once
			create Result.make ("item")
		end

invariant
	default_identities_name_exists: default_identities_name /= Void
	parent_name_exists: parent_name /= Void
	unknown_identity_name_exists: unknown_identity_name /= Void
	is_expanded_name_exists: is_expanded_name /= Void
	is_printable_name_exists: is_printable_name /= Void
	field_by_name_name_exists: field_by_name_name /= Void
	feature_names_unique: not unknown_identity_name.same_feature_name (is_expanded_name) and
		not unknown_identity_name.same_feature_name (is_printable_name) and
		not unknown_identity_name.same_feature_name (field_by_name_name) and
		not is_expanded_name.same_feature_name (is_printable_name) and
		not is_expanded_name.same_feature_name (field_by_name_name) and
		not is_printable_name.same_feature_name (field_by_name_name)
	routine_arguments_unique: not object_argument.same_identifier (field_name_argument)
end
