indexing
	description: "Objects that find program points of interest and process them"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_PROGRAM_POINT_FINDER

inherit
	CI_INSTRUCTION_COUNTER
		rename
			make as make_with_universe
		export {NONE}
			make_with_universe
		redefine
			process_class,
--			process_deferred_procedure,
--			process_deferred_function,
			process_do_procedure,
			process_do_function,
--			process_external_procedure,
--			process_external_function,
			process_once_procedure,
			process_once_function,
			process_loop_instruction,
			process_assignment,
			process_assignment_attempt,
			process_create_instruction,
			process_compound
		end

	CI_SETTINGS_CLIENT

	CI_CODE_GENERATION
		export {NONE}
			all
		end

create
	make

feature -- Initialization
	make (a_universe: like universe; a_printer: like printer) is
			-- Create program point finder that works within `a_universe' with program point information printer `a_printer'
		require
			a_universe_exists: a_universe /= Void
			a_printer_exists: a_printer /= Void
		do
			make_with_universe (a_universe)
			printer := a_printer
			create program_points.make (Settings.classes_to_instrument.count * 20)  -- Approx 10 features a class (?)
			program_points.compare_objects
			create loops.make
		ensure
			universe_set: universe = a_universe
			printer_set: printer = a_printer
		end

feature -- Access
	program_points: HASH_TABLE [CI_PROGRAM_POINT, STRING]
			-- All met program points

	printer: CI_PROGRAM_POINT_PRINTER
			-- Program point printer

feature -- Setting
	set_printer (a_printer: CI_PROGRAM_POINT_PRINTER) is
			-- Set `printer' to `a_printer'
		require
			a_printer_exist: a_printer /= Void
		do
			printer := a_printer
		ensure
			printer_set: printer = a_printer
		end

feature {ET_AST_NODE} -- Processing
	process_class (a_class: ET_CLASS) is
			-- Process child nodes, create and process object program point for `a_class'
		do
			current_class := a_class
			create current_stable_time.make (a_class, universe)
			Precursor (a_class)

			process_program_point (current_stable_time)
		end

--	process_deferred_procedure (a_routine: ET_DEFERRED_PROCEDURE) is
--			-- Process child nodes, create and process entry and exit program points for `a_routine'
--		do
--			current_procedure := a_routine
--			current_function := Void
--			process_procedure
--		end

--	process_deferred_function (a_routine: ET_DEFERRED_FUNCTION) is
--			-- Process child nodes, create and process entry and exit program points for `a_routine'
--		do
--			current_procedure := Void
--			current_function := a_routine
--			process_function
--		end

	process_do_procedure (a_routine: ET_DO_PROCEDURE) is
			-- Process child nodes, create and process entry and exit program points for `a_routine'
		do
			current_procedure := a_routine
			current_function := Void
			current_creator := creator (current_procedure, universe)

			Precursor (a_routine)
			process_procedure
			current_creator := Void
		end

	process_do_function (a_routine: ET_DO_FUNCTION) is
			-- Process child nodes, create and process entry and exit program points for `a_routine'
		do
			current_procedure := Void
			current_function := a_routine

			Precursor (a_routine)
			process_function
		end

--	process_external_procedure (a_routine: ET_EXTERNAL_PROCEDURE) is
--			-- Create and process entry and exit program points for `a_routine'
--		do
--			current_procedure := a_routine
--			current_function := Void
--			current_creator := creator

--			process_procedure
--			current_creator := Void
--		end

--	process_external_function (a_routine: ET_EXTERNAL_FUNCTION) is
--			-- Create and process entry and exit program points for `a_routine'
--		do
--			current_procedure := Void
--			current_function := a_routine
--			process_function
--		end

	process_once_procedure (a_routine: ET_ONCE_PROCEDURE) is
			-- Process child nodes, create and process entry and exit program points for `a_routine'
		do
			current_procedure := a_routine
			current_function := Void

			Precursor (a_routine)
			process_procedure
		end

	process_once_function (a_routine: ET_ONCE_FUNCTION) is
			-- Process child nodes, create and process entry and exit program points for `a_routine'
		do
			current_procedure := Void
			current_function := a_routine

			Precursor (a_routine)
			process_function
		end

	process_loop_instruction (an_instruction: ET_LOOP_INSTRUCTION) is
			-- Process child nodes, create and process loop program point for `an_instruction'
		local
			instr_count: INTEGER
			loop_point: CI_LOOP
			surrogate_loop: CI_SURROGATE_LOOP
			suppression: CI_LOOP_SUPPRESSION
			internal_procedure: ET_INTERNAL_PROCEDURE
			internal_function: ET_INTERNAL_FUNCTION
			modified_entities: LINKED_SET [ET_WRITABLE]
		do
			instr_count := instruction_count + 1
			create modified_entities.make
			modified_entities.compare_objects
			loops.extend (modified_entities)

			Precursor (an_instruction)

			loops.remove
			if not loops.is_empty then
				-- Add elements `modified_entities' to the set of modified entities of the surrounding loop
				from
					modified_entities.start
				until
					modified_entities.after
				loop
					loops.item.extend (modified_entities.item)
					modified_entities.forth
				end
			end

			check
				(current_class /= Void) and ((current_procedure /= Void) or (current_function /= Void))
					-- `client_class' was set in `process_class' before processing its child nodes
					-- `current_procedure' or `current_function' was set in `process_procedure' or `process_function'
			end

			internal_procedure ?= current_procedure
			internal_function ?= current_function
			if internal_procedure /= Void then
				create loop_point.make (an_instruction, instr_count, internal_procedure, current_class, universe)
			elseif internal_function /= Void then
				create loop_point.make (an_instruction, instr_count, internal_function, current_class, universe)
			end

			if Settings.loop_suppression_enabled and not Settings.use_old_decl_format then
				create surrogate_loop.make (modified_entities, loop_point)
				create suppression.make (surrogate_loop)
				loop_point.suppressions.extend (suppression)
				process_program_point (surrogate_loop)
			end
			if Settings.supplier_suppression_enabled then
				-- Has to be the last suppression in the list
				add_supplier_invariant_suppressions (loop_point)
			end
			process_program_point (loop_point)
		end

	process_assignment(an_instruction: ET_ASSIGNMENT) is
			-- Remember that the target of assingment is modified, if we are inside the loop
		do
			add_writable (an_instruction.target)
		end

	process_assignment_attempt (an_instruction: ET_ASSIGNMENT_ATTEMPT) is
			-- Remember that the target of assingment attempt is modified, if we are inside the loop
		do
			add_writable (an_instruction.target)
		end

	process_create_instruction (an_instruction: ET_CREATE_INSTRUCTION) is
			-- Remember that the target of creation instruction is modified, if we are inside the loop
		do
			add_writable (an_instruction.target)
		end

	process_compound (an_instruction: ET_COMPOUND) is
			-- Check wheter we are in a "from" of the loop or in a "loop body"
		do
			if not an_instruction.keyword.is_from then
				Precursor (an_instruction)
			end
		end

feature -- Implementation
	current_class: ET_CLASS
			-- Class that is currently processed

	current_procedure: ET_PROCEDURE
			-- Procedure that is currently processed

	current_function: ET_FUNCTION
			-- Function that is currently processed

	current_creator: ET_CREATOR
			-- Creator of `current_class' or its descendants, which lists `current_procedure'
			-- Void, if not a creation procedure

	current_stable_time: CI_STABLE_TIME
			-- Stable time program point corresponding to the invariant of `current_class'

	loops: LINKED_STACK [LINKED_SET [ET_WRITABLE]]
			-- Stack of sets of modified writables in the loop currently being processed and all its surrounding loops

	process_procedure is
			-- Create and process entry and exit program points for `current_procedure'
		require
			current_class_exists: current_class /= Void
			current_procedure_exists: current_procedure /= Void
		local
			entry: CI_ROUTINE_POINT
			exit: CI_ROUTINE_EXIT
			inv_suppression: CI_INVARIANT_SUPPRESSION
		do
			if current_creator /= Void then
				create {CI_CREATION_PROCEDURE_ENTRY} entry.make (current_procedure, current_creator.clients,
					current_class, universe)
			else
				create {CI_ROUTINE_ENTRY} entry.make (current_procedure, current_class, universe)
			end
			create exit.make (current_procedure, current_class, universe)

			if Settings.suppression_enabled then
				if current_creator /= Void then
					if not current_procedure.clients.is_none or not current_creator.clients.is_none then
						create inv_suppression.make (current_stable_time)
						exit.suppressions.extend (inv_suppression)
					end
				else
					if not current_procedure.clients.is_none then
						create inv_suppression.make (current_stable_time)
						entry.suppressions.extend (inv_suppression)
						exit.suppressions.extend (inv_suppression)
					end
				end
			end
			if Settings.supplier_suppression_enabled then
				-- Has to be the last suppression in the list
				add_supplier_invariant_suppressions (entry)
				add_supplier_invariant_suppressions (exit)
			end

			process_program_point (entry)
			process_program_point (exit)
		end

	process_function is
			-- Create and process entry and exit program points for `current_function'
		require
			current_class_exists: current_class /= Void
			current_function_exists: current_function /= Void
		local
			entry: CI_ROUTINE_ENTRY
			exit: CI_ROUTINE_EXIT
			inv_suppression: CI_INVARIANT_SUPPRESSION
			surrogate_exit: CI_SURROGATE_FUNCTION_EXIT
			function_suppression: CI_FUNCTION_SUPPRESSION
		do
			create entry.make (current_function, current_class, universe)
			create exit.make (current_function, current_class, universe)

			if Settings.suppression_enabled and not current_function.clients.is_none then
				create inv_suppression.make (current_stable_time)
				entry.suppressions.extend (inv_suppression)
				exit.suppressions.extend (inv_suppression)
			end

			if Settings.non_result_suppression_enabled and Settings.is_pure (current_function) and not Settings.use_old_decl_format then
				create surrogate_exit.make (current_function, current_class, universe)
				create function_suppression.make (surrogate_exit)
				exit.suppressions.extend (function_suppression)
				process_program_point (surrogate_exit)
			end

			if Settings.supplier_suppression_enabled then
				-- Has to be the last suppression in the list
				add_supplier_invariant_suppressions (entry)
				add_supplier_invariant_suppressions (exit)
			end

			process_program_point (entry)
			process_program_point (exit)
		end

	add_writable (a_writable: ET_WRITABLE) is
			-- Add `a_writable' to the top of `loops' if it exists
		do
			if not loops.is_empty then
				 -- We are processing a loop
				loops.item.extend (a_writable)
			end
		end

	add_supplier_invariant_suppressions (a_ppt: CI_PROGRAM_POINT) is
			-- Add supplier invariant suppressions to `a_ppt'
		require
			a_ppt_exists: a_ppt /= Void
		local
			stable_time: CI_STABLE_TIME
			cursor: DS_HASH_SET_CURSOR [ET_CLASS_NAME]
		do
			from
				cursor := Settings.classes_to_instrument.new_cursor
				cursor.start
			until
				cursor.after
			loop
				create stable_time.make (universe.eiffel_class (cursor.item), universe)
				a_ppt.suppressions.extend (create {CI_SUPPLIER_INVARIANT_SUPPRESSION}.make (stable_time))
				cursor.forth
			end
		end

	process_program_point (a_ppt: CI_PROGRAM_POINT) is
			-- Process `a_ppt'
		require
			a_ppt_exists: a_ppt /= Void
		do
			program_points.extend (a_ppt, printer.program_point_name (a_ppt))
		end

invariant
	program_points_exists: program_points /= Void
	printer_exists: printer /= Void
	can_not_process_procedure_and_function: (current_procedure = Void) or (current_function = Void)
	current_loop_stack_modified_entities_exists: loops /= Void
end
