indexing
	description: "Program points after specific instruction within body of specific internal routine%
		% (associated with check instruction)"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	CI_ROUTINE_BODY_POINT

inherit
	CI_ROUTINE_POINT
		rename
			make as make_with_routine_and_class
		export {NONE}
			make_with_routine_and_class
		redefine
			surrounding_routine,
			add_variables,
			state_printing_function_formals,
			state_printing_function_actuals
		end

feature -- Initialization
	make (a_position: like position; a_routine: like surrounding_routine;
		a_class: like surrounding_class; a_universe: like universe) is
			-- Create program point after instruction `position' in routine `a_routine' of class `a_class' in universe `a_universe'
		require
			a_position_not_too_small: a_position >= 0
			-- a_position_not_too_large: a_position <= instruction_count (a_routine)
			a_class_exists: a_class /= Void
			a_universe_exists: a_universe /= Void
			a_routine_exists: a_routine /= Void
		do
			position := a_position
			make_with_routine_and_class (a_routine, a_class, a_universe)
		ensure
			position_set: position = a_position
			surrounding_routine_set: surrounding_routine = a_routine
			surrounding_class_set: surrounding_class = a_class
			universe_set: universe = a_universe
		end

feature -- Access
	surrounding_routine: ET_INTERNAL_ROUTINE
			-- Routine to which program point belongs
			-- Only internal routines have body, within which assertions may be checked

	position: INTEGER
			-- After which instruction does program point appear in routine body?
			-- Value 0 means before any instruction

	kind (a_visitor: CI_PROGRAM_POINT_KIND_FORMAT): STRING is
			-- Program point kind name
		do
			Result := a_visitor.body_kind
		end

feature {NONE} -- Implementation
	add_variables is
			-- Find variables in scope and store them into `variables'
			-- Add local entities to variables found by `Precursor'
		local
			loc: CI_LOCAL_ENTITY
			res: CI_RESULT
			i: INTEGER
		do
			Precursor

			if surrounding_routine.locals /= Void then
				from
					i := 1
				until
					i > surrounding_routine.locals.count
				loop
					-- Add a local
					create loc.make (surrounding_routine.locals.local_variable (i),
						surrounding_routine, surrounding_class, universe)
					-- Add `surrounding_class' to local's `clients'
					loc.clients.force_first (surrounding_class.name)
					variables.put (loc)
					i := i + 1
				end
			end
			if is_function_point then
				create res.make (surrounding_function, surrounding_class, universe)
				-- Add `surrounding_class' to Result's clients
				res.clients.force_first (surrounding_class.name)
				variables.put (res)
			end
		end

feature {NONE} -- Assertions
	state_printing_function_formals: ET_FORMAL_ARGUMENT_LIST is
			-- Formal arguments of `state_printing_function'
		local
			locs: ET_LOCAL_VARIABLE_LIST
			i: INTEGER
		do
			Result := Precursor
			locs := surrounding_routine.locals
			if locs /= Void then
				if Result = Void then
					create Result.make_with_capacity (locs.count)
				else
					Result.resize (Result.count + locs.count)
				end
				from
					i := locs.count
				until
					i < 1
				loop
					Result.put_first (
						create {ET_FORMAL_ARGUMENT}.make (locs.item (i).name, locs.local_variable (i).declared_type))
					i := i - 1
				end
			end
			if is_function_point then
				if Result = Void then
					create Result.make_with_capacity (1)
				end
				Result.force_first (create {ET_FORMAL_ARGUMENT}.make (corrupted (tokens.result_keyword.text),
					surrounding_function.declared_type))
			end
		end

	state_printing_function_actuals: ET_ACTUAL_ARGUMENT_LIST is
			-- Actual arguments of `state_printing_function' call inside instrumented `checked_assertions'
		local
			locs: ET_LOCAL_VARIABLE_LIST
			i: INTEGER
		do
			Result := Precursor
			locs := surrounding_routine.locals
			if locs /= Void then
				if Result = Void then
					create Result.make_with_capacity (locs.count)
				else
					Result.resize (Result.count + locs.count)
				end
				from
					i := locs.count
				until
					i < 1
				loop
					Result.put_first (locs.item (i).name)
					i := i - 1
				end
			end
			if is_function_point then
				if Result = Void then
					create Result.make_with_capacity (1)
				end
				Result.force_first (tokens.result_keyword)
			end
		end

invariant
	position_not_too_small: position >= 0
	-- position_not_too_large: position <= instruction_count (surrounding_routine)
end
