indexing
	description: "Program points that belong to specific routine"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	CI_ROUTINE_POINT

inherit
	CI_PROGRAM_POINT
		rename
			make as make_with_class
		export {NONE}
			make_with_class
		redefine
			add_variables,
			is_function_point
		end

feature -- Initialization
	make (a_routine: like surrounding_routine; a_class: like surrounding_class; a_universe: like universe) is
			-- Create a program point within `a_routine' of `a_class' in `a_universe'
		require
			a_routine_exists: a_class /= Void
			a_class_exists: a_class /= Void
			a_universe_exists: a_universe /= Void
		do
			surrounding_routine := a_routine
			make_with_class (a_class, a_universe)
		ensure
			surrounding_routine_set: surrounding_routine = a_routine
			surrounding_class_set: surrounding_class = a_class
			universe_set: universe = a_universe
		end

feature -- Status report
	is_function_point: BOOLEAN is
			-- Does the point belong to a function (rather than a procedure)?
		do
			Result := surrounding_routine.is_function
		end

feature {NONE} -- Implementation
	add_variables is
			-- Find variables in scope and store them into `variables'
			-- Add formal arguments to variables found by `Precursor'
		local
			cur: CI_CURRENT
			var: CI_FORMAL_ARGUMENT
			i: INTEGER
		do
			create cur.make (surrounding_routine, surrounding_class, universe)
			variables.put (cur)
			from
				i := 1
			until
				i > surrounding_routine.arguments_count
			loop
				-- Add formal
				create var.make (surrounding_routine.arguments.formal_argument (i), surrounding_routine, surrounding_class, universe)
				-- Add `surrounding class' to clients of formal
				var.clients.force_first (surrounding_class.name)
				variables.put (var)
				i := i + 1
			end
			add_constants
		end

feature {NONE} -- Assertions
	state_printing_function_formals: ET_FORMAL_ARGUMENT_LIST is
			-- Formal arguments of `state_printing_function'
		do
			Result :=  formals_twin (surrounding_routine.arguments)
		end

	state_printing_function_actuals: ET_ACTUAL_ARGUMENT_LIST is
			-- Actual arguments of `state_printing_function' call inside instrumented `checked_assertions'
		local
			args: ET_FORMAL_ARGUMENT_LIST
			i: INTEGER
		do
			args := surrounding_routine.arguments
			if args /= Void then
				create Result.make_with_capacity (args.count)
				from
					i := args.count
				until
					i < 1
				loop
					Result.put_first (args.item (i).name)
					i := i - 1
				end
			end
		end

invariant
	surrounding_routine_exits: surrounding_routine /= Void
	surrounding_function_exists_when_function_point: is_function_point implies surrounding_function /= Void
end
