indexing
	description: "Objects that generate code needed to use RUNTIME_OUTPUT features"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_RUNTIME_OUTPUT_METACLIENT

inherit
	CI_RUNTIME_OUTPUT_NAMES_CLIENT

	CI_SETTINGS_CLIENT

	CI_CODE_GENERATION
		export {NONE}
			all
		end

feature -- Basic operations
	update_parents (a_class: ET_CLASS; a_universe: ET_UNIVERSE) is
			-- Update parents of `a_class' from `a_universe' so that it will have access to "RUNTIME_OUTPUT" singleton instance
		require
			a_class_exists: a_class /= Void
			a_universe_exists: a_universe /= Void
		local
			i: INTEGER
			parents: ET_PARENT_LIST
			f, new_f: ET_FEATURE
			renames: ET_RENAME_LIST
			runtime_output_client_type: ET_CLASS_TYPE
			runtime_output_client_parent, any_parent: ET_PARENT
		do
			if not Settings.is_any_changed then
				create runtime_output_client_type.make (Void,
					RO_names.runtime_output_client_name,
					a_universe.eiffel_class (RO_names.runtime_output_client_name))
				if a_class.parents = Void then
					create runtime_output_client_parent.make (runtime_output_client_type, Void, Void, Void, Void, Void)
					create any_parent.make (a_universe.any_type, Void, Void, Void, Void, Void)
					create parents.make_with_capacity (2)
					parents.put_first (runtime_output_client_parent)
					parents.put_first (any_parent)
					a_class.set_parents (parents)
				else
					create renames.make
					from i := a_universe.any_class.queries.declared_count until i < 1 loop
						f := a_universe.any_class.queries.item (i)
						new_f := a_class.seeded_query (f.first_seed)
						check feature_from_any: new_f /= Void end
						if not new_f.name.same_feature_name (f.name) then
							renames.force_first (create {ET_RENAME}.make (f.name, new_f.extended_name))
						end
						i := i - 1
					end
					from i := a_universe.any_class.procedures.declared_count until i < 1 loop
						f := a_universe.any_class.procedures.item (i)
						new_f := a_class.seeded_procedure (f.first_seed)
						check feature_from_any: new_f /= Void end
						if not new_f.name.same_feature_name (f.name) then
							renames.force_first (create {ET_RENAME}.make (f.name, new_f.extended_name))
						end
						i := i - 1
					end
					if renames.is_empty then
						create runtime_output_client_parent.make (runtime_output_client_type, Void, Void, Void, Void, Void)
					else
						create runtime_output_client_parent.make (runtime_output_client_type, renames, Void, Void, Void, Void)
					end

					a_class.parents.force_first (runtime_output_client_parent)
				end
			end
		end

feature -- Access
	trace_expression: ET_EXPRESSION is
			-- Expression that yields "RUNTIME_OUTPUT" instance inside "RUNTIME_OUTPUT_CLIENT" descendant
		once
			create {ET_CALL_EXPRESSION} Result.make (Void, RO_names.trace_feature_name, Void)
		end

	file_expression: ET_EXPRESSION is
			-- Expression that yields output file inside "RUNTIME_OUTPUT_CLIENT" descendant
		once
			create {ET_CALL_EXPRESSION} Result.make (trace_expression, RO_names.file_feature_name, Void)
		end

	variable_printing_instruction (a_name: STRING; a_value: ET_EXPRESSION; a_modified: STRING): ET_INSTRUCTION is
			-- Instruction that calls "print_var" with appropriate arguments from inside "RUNTIME_OUTPUT_CLIENT" descendant
		require
			a_name_exists: a_name /= Void
			a_value_exists: a_value /= Void
			a_modified_exists: a_modified /= Void
		local
			args: ET_ACTUAL_ARGUMENT_LIST
		do
			create args.make_with_capacity (3)
			args.put_first (manifest_integer (a_modified))
			args.put_first (a_value)
			args.put_first (manifest_string (a_name))
			create {ET_CALL_INSTRUCTION} Result.make (trace_expression, RO_names.print_var_feature_name, args)
		ensure
			result_exists: Result /= Void
		end

	string_printing_instruction (a_string: STRING): ET_INSTRUCTION is
			-- Instruction that prints `a_string' from inside "RUNTIME_OUTPUT_CLIENT" descendant
		require
			a_string_exists: a_string /= Void
		do
			create {ET_CALL_INSTRUCTION} Result.make (file_expression, tokens.put_string_feature_name,
				single_actual (manifest_string (a_string)))
		ensure
			result_exists: Result /= Void
		end

	printing_instruction (an_expression: ET_EXPRESSION) : ET_INSTRUCTION is
			-- Instruction that prints `an_expression' from inside "RUNTIME_OUTPUT_CLIENT" descendant
		require
			an_expression_exists: an_expression /= Void
		local
			arg: ET_CALL_EXPRESSION
		do
			create arg.make (parenthesized_for_dot (an_expression), tokens.out_feature_name, Void)
			create {ET_CALL_INSTRUCTION} Result.make (file_expression, tokens.put_string_feature_name, single_actual (arg))
		ensure
			result_exists: Result /= Void
		end

	new_line_instruction: ET_INSTRUCTION is
			-- An instruction that prints a new line from inside "RUNTIME_OUTPUT_CLIENT" descendant
		once
			create {ET_CALL_INSTRUCTION} Result.make (file_expression, tokens.new_line_feature_name, Void)
		end

--	flush_instruction: ET_INSTRUCTION is
--			-- An instruction to flush buffered runtime output from inside "RUNTIME_OUTPUT_CLIENT" descendant
--		once
--			create {ET_CALL_INSTRUCTION} Result.make (file_expression, tokens.flush_feature_name, Void)
--		end

	called_expression (a_class_name: ET_CLASS_NAME; a_feature_name: ET_FEATURE_NAME): ET_CALL_EXPRESSION is
			-- Expression that yield result of "<class>_<feature>_called" function
		require
			a_class_name_exists: a_class_name /= Void
			a_feature_name_exists: a_feature_name /= Void
		do
			create Result.make (trace_expression, RO_names.called_feature_name (a_class_name, a_feature_name), Void)
		ensure
			result_exists: Result /= Void
		end

	set_called_instruction (a_class_name: ET_CLASS_NAME; a_feature_name: ET_FEATURE_NAME): ET_CALL_INSTRUCTION is
			-- Instruction that calls "set_<class>_<feature>_called" procedure
		require
			a_class_name_exists: a_class_name /= Void
			a_feature_name_exists: a_feature_name /= Void
		do
			create Result.make (trace_expression, RO_names.set_called_feature_name (a_class_name, a_feature_name), Void)
		ensure
			result_exists: Result /= Void
		end

invariant
	file_expression_exists: file_expression /= Void
	new_line_instruction_exists: new_line_instruction /= Void
--	flush_instruction_exists: flush_instruction /= Void
end
