indexing
	description: "Objects that represent fields and their values in Daikon declaration files"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_FIELD

inherit
	ANY
		redefine
			out
		end

create
	make

feature -- Initialization
	make (n, v: STRING; i: INTEGER) is
			-- Create a fields with name `n', value `v' and identation `i'
		require
			n_non_empty: n /= Void and then not n.is_empty
			v_non_empty: v /= Void and then not v.is_empty
			i_non_ngeative: i >=0
		do
			name := n
			value := v
			indentation := i
		ensure
			name_set: name = n
			value_set: value = v
			identation_set: indentation = i
		end

feature -- Access
	name: STRING
			-- Field name

	value: STRING
			-- Field value

	indentation: INTEGER
			-- Indentation for a field

feature -- Output
	out: STRING is
			-- String representation of field in the decls file (with new line)
		local
			i: INTEGER
		do
			create Result.make_empty
			from
				i := 1
			until
				i > indentation
			loop
				Result.append_character ('%T')
				i := i + 1
			end
			Result.append (name + " " + value + "%N")
		end

invariant
	name_not_empty: not name.is_empty
	value_not_empty: not value.is_empty
	indentation_non_negative: indentation >= 0
end
