indexing
	description: "Printable type kinds"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CI_PRINTABLE

inherit
	CI_VARIABLE_TYPE_KIND
		redefine
			is_printable,
			children
		end

	CI_IDENTITIES_NAMES_CLIENT

	CI_RUNTIME_OUTPUT_METACLIENT
		export {NONE}
			all
		end
create
	make

feature -- Access
	identity (a_variable: CI_VARIABLE): ET_EXPRESSION is
			-- Expression with printable result that identifies value of printable variable `a_variable'
		local
			i: INTEGER
		do
			i := printable_classes.index_of (a_variable.type.base_class.name, 1)
			if boolean_classes.has (i) then
				-- Boolean values should be output as {0, 1}
				create {ET_CALL_EXPRESSION} Result.make (parenthesized_for_dot (a_variable.as_expression),
					tokens.to_integer_feature_name, Void)
			elseif character_classes.has (i) then
				-- Characters should be output as integers
				create {ET_CALL_EXPRESSION} Result.make (parenthesized_for_dot (a_variable.as_expression),
					tokens.code_feature_name, Void)
			elseif string_classes.has (i) then
				-- String should be quoted and without new lines
				create {ET_CALL_EXPRESSION} Result.make (trace_expression, RO_names.string_rep_feature_name,
					single_actual (a_variable.as_expression))
			else
				Result := a_variable.as_expression
			end
		end

	uniform_identity (an_expression: ET_IDENTIFIER): ET_EXPRESSION is
			-- Identity expression that has the same type (`INTEGER') independently of `base_type'
		do
--			create {ET_CALL_EXPRESSION} Result.make (an_expression, tokens.hash_code_feature_name, Void)
			create {ET_CALL_EXPRESSION} Result.make (Void, Identities_names.printable_hash_code_name, single_actual (an_expression))
		end

	uniform_identity_argument_type: ET_TYPE is
			-- Reference type, which is a valid "argument" type for `uniform_identity'
		once
			Result := type_from_nongeneric_class (universe.hashable_class)
		end

	children (a_variable: CI_VARIABLE): LIST [CI_VARIABLE] is
			-- Child variables of `a_variable'
		do
			if Settings.unfold_printable_variables then
				Result := Precursor (a_variable)
			else
				create {LINKED_LIST [CI_VARIABLE]} Result.make
			end
		end

	representation_type (a_class_name: ET_CLASS_NAME): STRING is
			-- Name of Daikon representation type for printable type based on class called `a_class_name'
		do
			Result := daikon_type (a_class_name)
		end

feature -- Status report
	is_printable: BOOLEAN is
			-- Is type printable?
		once
			Result := True
		end
end
