indexing
	description: "Kinds of trackable variables types that differ in variable identity representation"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	CI_VARIABLE_TYPE_KIND

inherit
	ANY

	CI_CODE_GENERATION
		export {NONE}
			all
		end

	CI_PRINTABLE_CLASSES
		rename
			is_printable as is_printable_class
		export {NONE}
			all
		end

feature -- Initialization
	make (a_universe: like universe) is
			-- Create type kind within `a_universe'
		require
			a_universe_exists: a_universe /= Void
		do
			universe := a_universe
		ensure
			universe_set: universe = a_universe
		end

feature -- Access
	identity (a_variable: CI_VARIABLE): ET_EXPRESSION is
			-- Expression with printable result that identifies value of `a_variable' with type kind `Current'
		require
			kind_correct: a_variable /= Void and then a_variable.type.kind.is_equal (Current)
		deferred
		ensure
			result_exists: Result /= Void
		end

	uniform_identity (an_expression: ET_IDENTIFIER): ET_EXPRESSION is
			-- Identity expression that has the same type (`INTEGER') independently of variable type
		require
			an_expression_exists: an_expression /= Void
		deferred
		ensure
			result_exists: Result /= Void
		end

	uniform_identity_argument_type: ET_TYPE is
			-- Reference type, which is a valid "argument" type for `uniform_identity'
		deferred
		end

	uniform_identity_type_size: INTEGER is 32
			-- Size of `uniform_identity' result type

	children (a_variable: CI_VARIABLE): LIST [CI_VARIABLE] is
			-- Child variables of `a_variable'
		require
			a_variable_exists: a_variable /= Void
			a_variable_has_this_kind: a_variable.type.kind = Current
		local
			i: INTEGER
			queries: ET_QUERY_LIST
			query: ET_QUERY
			query_var: CI_QUERY
		do
			create {LINKED_LIST [CI_VARIABLE]} Result.make
			queries := a_variable.type.base_class.queries
			from
				i := 1
			until
				i > queries.count
			loop
				query := queries.item (i)
				if a_variable.valid_child (query) then
					create {CI_QUERY} query_var.make (query, a_variable)
					if not (query.is_constant_attribute and not query_var.type.kind.is_string_reference) then
						-- not expanded constant attribute
						Result.extend (query_var)
					end
				end
				i := i + 1
			end
		ensure
			result_exists: Result /= Void
		end

	representation_type (a_class_name: ET_CLASS_NAME): STRING is
			-- Name of Daikon representation type for type with current kind based on class called `a_class_name'
		require
			a_class_name_exists: a_class_name /= Void
		deferred
		ensure
			result_not_empty: not result.is_empty
		end

	universe: ET_UNIVERSE
			-- Surrounding universe

feature -- Status report
	is_printable: BOOLEAN is
			-- Is type printable?
		once
			-- By default answer is "No"			
		end

	is_reference: BOOLEAN is
			-- Is type reference?
		once
			-- By default answer is "No"			
		end

	is_string_reference: BOOLEAN is
			-- Is type string reference?
		once
			-- By default answer is "No"			
		end

	is_expanded: BOOLEAN is
			-- Is type expanded?
		once
			-- By default answer is "No"			
		end

	uses_address: BOOLEAN is
			-- Is variable `address_expression' used in its `identity'?
		once
			-- By default answer is "No"						
		end

invariant
	universe_exists: universe /= Void
	uniform_identity_argument_type_exists: uniform_identity_argument_type /= Void
end
