indexing
	description: "Command line parsers for CITADEL"
	author: "Nadia Polikarpova"
	date: "$Date$"
	revision: "$Revision$"

class
	CITADEL_COMMAND_LINE_PARSER

inherit
	CI_SETTINGS_CLIENT

	CI_ERROR_HANDLER_CLIENT

	KL_SHARED_EXCEPTIONS

	KL_SHARED_ARGUMENTS

	KL_SHARED_FILE_SYSTEM

create
	make_null

feature -- Parsing
	process_arguments is
			-- Process arguments.
		require
			error_hanlder_not_void: error_handler /= Void
		local
			parser: AP_PARSER
			version_option: AP_FLAG
			quiet_option: AP_FLAG
			system_target_option: AP_STRING_OPTION
			cluster_to_instrument_option: AP_STRING_OPTION
			class_to_instrument_option: AP_STRING_OPTION
			output_dir_option: AP_STRING_OPTION
			start_with_option: AP_INTEGER_OPTION
			end_with_option: AP_INTEGER_OPTION
			unfolding_depth_option: AP_INTEGER_OPTION
			only_attributes_option: AP_FLAG
			unfold_printable_variables_option: AP_FLAG
			use_any_queries_option: AP_FLAG
			use_obsolete_queries_option: AP_FLAG
			use_inapplicable_queries_option: AP_FLAG
			once_status_mode_option: AP_STRING_OPTION
			finalize_option: AP_FLAG
			instrumented_system_args_option: AP_STRING_OPTION
			daikon_args_option: AP_STRING_OPTION
			keep_original_contracts_option: AP_STRING_OPTION
			first, last: INTEGER
			requested_steps: INTEGER_INTERVAL
			mode: INTEGER
			use_old_decl_file_format_option: AP_FLAG
			impure_function_option: AP_STRING_OPTION
			suppression_option: AP_FLAG
			non_result_suppression_option: AP_FLAG
			loop_suppression_option: AP_FLAG
			supplier_suppression_option: AP_FLAG
			java_memory_option: AP_INTEGER_OPTION
		do
			create parser.make
			parser.set_error_handler (error_handler)

			parser.set_application_description ("CITADEL (Contract Inference Tool that Applies Daikon to Eiffel Language) %
				%is an Eiffel font-end for Daikon invariant detector.")
			parser.set_parameters_description ("ecf_filename")

			create version_option.make ('v', "version")
			version_option.set_description ("Output version information and exit.")
			parser.alternative_options_lists.force_last (create {AP_ALTERNATIVE_OPTIONS_LIST}.make (version_option))

			create quiet_option.make ('q', "quiet")
			quiet_option.set_description ("Be quiet.")
			parser.options.force_last (quiet_option)

			create system_target_option.make ('t', "target")
			system_target_option.set_description ("Target (from supplied ECF file) that should be used for contract inference.")
			parser.options.force_last (system_target_option)

			create cluster_to_instrument_option.make ('l', "cluster")
			cluster_to_instrument_option.set_description ("Cluster to instrument. May be used multiple times. Use" +
				Settings.subcluster_separator.out + " as subcluster name separator. If no clusters are specified, %
				%all clusters are chosen for instrumentation.")
			parser.options.force_last (cluster_to_instrument_option)

			create class_to_instrument_option.make ('c', "class")
			class_to_instrument_option.set_description ("Class to instrument. May be used multiple times. If no classes from chosen %
				%cluster are specified, all classes are chosen for instrumentation.")
			parser.options.force_last (class_to_instrument_option)

			create output_dir_option.make ('o', "output-dir")
			output_dir_option.set_description ("Output directory for instrumented system and Daikon files.")
			parser.options.force_last (output_dir_option)

			create start_with_option.make ('s', "start")
			start_with_option.set_description ("Step to start with " + steps_description + ". Default is " + steps.lower.out + ".")
			parser.options.force_last (start_with_option)

			create end_with_option.make ('e', "end")
			end_with_option.set_description ("Step to end with " + steps_description + ". Default is " + steps.upper.out + ".")
			parser.options.force_last (end_with_option)

			create use_old_decl_file_format_option.make ('m', "use-old-decl-format")
			use_old_decl_file_format_option.set_description ("Use the old version of the daikon declaration file format")
			parser.options.force_last (use_old_decl_file_format_option)

			create unfolding_depth_option.make ('u', "unfolding-depth")
			unfolding_depth_option.set_description ("Depth to which variables are unfolded. Default is " +
				{CI_VARIABLE}.default_unfolding_depth.out + ".")
			parser.options.force_last (unfolding_depth_option)

			create impure_function_option.make ('g', "impure")
			impure_function_option.set_description ("Name of impure function that should not be used in unfolding in format CLASS_NAME" + Settings.Class_feature_separator + "function_name")
			parser.options.force_last (impure_function_option)

			create only_attributes_option.make ('a', "only-attributes")
			only_attributes_option.set_description ("Do not use functions in unfolding, only attributes.")
			parser.options.force_last (only_attributes_option)

			create unfold_printable_variables_option.make ('p', "unfold-printable")
			unfold_printable_variables_option.set_description ("Unfold printable variables (may significantly enlarge output,%
				% but gives some extra information about objects of printable types).")
			parser.options.force_last (unfold_printable_variables_option)

			create use_any_queries_option.make ('y', "any-queries")
			use_any_queries_option.set_description ("Use queries, inherited from class ANY, in unfolding (may significantly enlarge output).")
			parser.options.force_last (use_any_queries_option)

			create use_obsolete_queries_option.make ('b', "obsolete-queries")
			use_obsolete_queries_option.set_description ("Use obsolete queries in unfolding.")
			parser.options.force_last (use_obsolete_queries_option)

			create use_inapplicable_queries_option.make ('z', "inapplicable-queries")
			use_inapplicable_queries_option.set_description ("Use inapplicable queries in unfolding.")
			parser.options.force_last (use_inapplicable_queries_option)

			create once_status_mode_option.make ('n', "once-status")
			once_status_mode_option.set_description (once_status_mode_description)
			parser.options.force_last (once_status_mode_option)

			create finalize_option.make ('f', "finalize")
			finalize_option.set_description ("Finalize instrumented system. (Better performance, but no melting).")
			parser.options.force_last (finalize_option)

			create instrumented_system_args_option.make ('i', "instrumented-args")
			instrumented_system_args_option.set_description ("Arguments to pass to instrumented system.")
			parser.options.force_last (instrumented_system_args_option)

			create daikon_args_option.make ('d', "daikon-args")
			daikon_args_option.set_description ("Arguments to pass to Daikon.")
			parser.options.force_last (daikon_args_option)

			create java_memory_option.make ('j', "java-memory")
			java_memory_option.set_description ("Memory size for JVM when running Daikon and PrintInvariants")
			parser.options.force_last (java_memory_option)

			create suppression_option.make ('r', "disable-suppression")
			suppression_option.set_description ("Disable pre- and postcondition suppression by the class invariant")
			parser.options.force_last (suppression_option)

			create non_result_suppression_option.make_with_long_form ("disable-nonres-suppression")
			non_result_suppression_option.set_description ("Disable suppression of pure function postconditions not involving Result")
			parser.options.force_last (non_result_suppression_option)

			create loop_suppression_option.make_with_long_form ("disable-loop-suppression")
			loop_suppression_option.set_description ("Disable suppression of loop invariants not involving variables modified in the loop body")
			parser.options.force_last (loop_suppression_option)

			create supplier_suppression_option.make_with_long_form ("disable-supplier-suppression")
			supplier_suppression_option.set_description ("Disable suppression of clauses by class invariants of the involved variables")
			parser.options.force_last (supplier_suppression_option)

			create keep_original_contracts_option.make ('k', "keep-original")
			keep_original_contracts_option.set_description (keep_original_contracts_description)
			parser.options.force_last (keep_original_contracts_option)

			parser.parse_arguments

			if version_option.was_found then
				error_handler.enable_verbose
				error_handler.report_version
				error_handler.disable_verbose
				Exceptions.die (0)
			end

			if not quiet_option.was_found then
				error_handler.enable_verbose
			end

			if system_target_option.was_found then
				Settings.set_system_target (system_target_option.parameter)
			end

			if output_dir_option.was_found then
				Settings.set_output_path (output_dir_option.parameter)
			end

			if cluster_to_instrument_option.was_found then
				from
					cluster_to_instrument_option.parameters.start
				until
					cluster_to_instrument_option.parameters.after
				loop
					Settings.clusters_to_instrument.force (cluster_to_instrument_option.parameters.item_for_iteration)
					cluster_to_instrument_option.parameters.forth
				end
			end

			if class_to_instrument_option.was_found then
				from
					class_to_instrument_option.parameters.start
				until
					class_to_instrument_option.parameters.after
				loop
					Settings.classes_to_instrument.force (
						create {ET_IDENTIFIER}.make (class_to_instrument_option.parameters.item_for_iteration))
					class_to_instrument_option.parameters.forth
				end
			end

			if start_with_option.was_found then
				first := start_with_option.parameter
			else
				first := steps.lower
			end
			if end_with_option.was_found then
				last := end_with_option.parameter
			else
				last := steps.upper
			end
			if first < steps.lower or last > steps.upper or first > last then
				error_handler.report_incorrect_steps (first, last)
			else
				create requested_steps.make (first, last)
				if requested_steps.has (instrumentation_index) then Settings.set_instrumentation_enabled (True) end
				if requested_steps.has (compilation_index) then Settings.set_compilation_enabled (True)	end
				if requested_steps.has (execution_index) then Settings.set_execution_enabled (True) end
				if requested_steps.has (inference_index) then Settings.set_inference_enabled (True) end
				if requested_steps.has (annotation_index) then Settings.set_annotation_enabled (True) end
			end

			if unfolding_depth_option.was_found then
				if unfolding_depth_option.parameter >= 0 then
					Settings.set_unfolding_depth (unfolding_depth_option.parameter)
				else
					error_handler.report_incorrect_unfolding_depth (unfolding_depth_option.parameter)
				end
			end
			Settings.set_use_functions (not only_attributes_option.was_found)
			Settings.set_unfold_printable_variables (unfold_printable_variables_option.was_found)
			Settings.set_use_any_queries (use_any_queries_option.was_found)
			Settings.set_use_obsolete_queries (use_obsolete_queries_option.was_found)
			Settings.set_use_obsolete_queries (use_inapplicable_queries_option.was_found)

			if once_status_mode_option.was_found then
				mode := mode_index (once_status_mode_option.parameter, once_status_modes)
				if mode > once_status_modes.upper then
					error_handler.report_incorrect_once_status_mode (once_status_mode_option.parameter)
				else
					Settings.set_once_status_mode (mode)
				end
			end

			Settings.set_workbench_mode (not finalize_option.was_found)

			if instrumented_system_args_option.was_found then
				Settings.set_instrumented_system_arguments (instrumented_system_args_option.parameter)
			end

			if daikon_args_option.was_found then
				Settings.set_daikon_arguments (daikon_args_option.parameter)
			end

			Settings.set_suppression_enabled (not suppression_option.was_found)
			Settings.set_non_result_suppression_enabled (not non_result_suppression_option.was_found)
			Settings.set_loop_suppression_enabled (not loop_suppression_option.was_found)
			Settings.set_supplier_suppression_enabled (not supplier_suppression_option.was_found)

			if keep_original_contracts_option.was_found then
				mode := mode_index (keep_original_contracts_option.parameter, keep_original_contracts_modes)
				if mode > keep_original_contracts_modes.upper then
					error_handler.report_incorrect_keep_mode (keep_original_contracts_option.parameter)
				else
					Settings.set_keep_original_contracts_mode (mode)
				end
			end

			settings.set_use_old_decl_format (use_old_decl_file_format_option.was_found)

			if impure_function_option.was_found then
				from
					impure_function_option.parameters.start
				until
					impure_function_option.parameters.after
				loop
					Settings.impure_functions.force (impure_function_option.parameters.item_for_iteration)
					impure_function_option.parameters.forth
				end
			end

			if parser.parameters.count = 0 then
				error_handler.report_missing_ecf_filename_error
			else
				if file_system.file_exists (parser.parameters.first) then
					Settings.set_system_name (parser.parameters.first)
				else
					error_handler.report_error_message ("Configuration file " + parser.parameters.first + " doesn't exist")
				end
			end

			if java_memory_option.was_found then
				Settings.set_java_memory_size (java_memory_option.parameter)
			end
		end

feature {NONE} -- Implementation
	instrumentation_index: INTEGER is 1
	compilation_index: INTEGER is 2
	execution_index: INTEGER is 3
	inference_index: INTEGER is 4
	annotation_index: INTEGER is 5

	steps: ARRAY [STRING] is
			-- Steps that CITADEL execution consists of
		once
			create Result.make (instrumentation_index, annotation_index)
			Result [instrumentation_index] := "instrumentation"
			Result [compilation_index] := "compilation"
			Result [execution_index] := "execution"
			Result [inference_index] := "inference"
			Result [annotation_index] := "annotation"
		end

	steps_description: STRING is
			-- Description of steps for users
		local
			i: INTEGER
		once
			Result := "("
			from
				i := steps.lower
			until
				i > steps.upper
			loop
				Result.append_integer (i)
				Result.append (" = ")
				Result.append (steps [i])
				Result.append_character (' ')
				i := i + 1
			end
			Result.append_character (')')
		end

	once_status_modes: ARRAY [STRING] is
			-- String reporesentations of once status modes
		once
			create Result.make (Settings.none_mode, Settings.unsafe_mode)
			Result [Settings.none_mode] := "none"
			Result [Settings.incomplete_mode] := "incomplete"
			Result [Settings.slow_mode] := "slow"
			Result [Settings.unsafe_mode] := "unsafe"
			Result.compare_objects
		end

	once_status_mode_description: STRING is
			-- Description of `once_status_mode_option' for users
		once
			Result :=  "How once functions status (called or not yet) is taken into account in unfolding (" +
			once_status_modes [Settings.none_mode] + " = once functions are not used in unfolding; " +
			once_status_modes [Settings.incomplete_mode] + " = only once functions from instrumented classes are used in unfolding; " +
			once_status_modes [Settings.slow_mode] + " = all once functions are used, but requires modification of all system, not just instrumented classes; " +
			once_status_modes [Settings.unsafe_mode] + " = once functions are used without watching their status (can lead to system behavior alteration)). Default is "
			+ once_status_modes [Settings.once_status_default_mode] + "."
		end

	keep_original_contracts_modes: ARRAY [STRING] is
			-- String reporesentations of once status modes
		once
			create Result.make (Settings.remove_mode, Settings.flatten_mode)
			Result [Settings.remove_mode] := "remove"
			Result [Settings.keep_mode] := "keep"
			Result [Settings.flatten_mode] := "flatten"
			Result.compare_objects
		end

	keep_original_contracts_description: STRING is
			-- Description of `once_status_mode_option' for users
		once
			Result :=  "Are original assertions kept in annotated system? (" +
			keep_original_contracts_modes [Settings.remove_mode] + "; " +
			keep_original_contracts_modes [Settings.keep_mode] + "; " +
			keep_original_contracts_modes [Settings.flatten_mode] + ") Default is "
			+ keep_original_contracts_modes [Settings.keep_original_contracts_default_mode] + "."
		end

	mode_index (a_str: STRING; a_modes: ARRAY [STRING]): INTEGER is
			-- Index of `a_str' in `a_modes'
		local
			i: INTEGER
		do
			from
				i := a_modes.lower
			until
				i > a_modes.upper
			loop
				if a_modes [i].is_equal (a_str) then
					Result := i
					i := a_modes.upper
				end
				i := i + 1
			end
		end

end
